@@ -761,7 +761,7 @@ g_match_info_fetch_all (const GMatchInfo *match_info)
   gchar **result;
   gint i;
 
-  g_return_val_if_fail (match_info != NULL, FALSE);
+  g_return_val_if_fail (match_info != NULL, NULL);
 
   if (match_info->matches < 0)
     return NULL;
@@ -858,6 +858,7 @@ g_regex_new (const gchar         *pattern,
   gint erroffset;
   gboolean optimize = FALSE;
   static gboolean initialized = FALSE;
+  unsigned long int pcre_compile_options;
 
   g_return_val_if_fail (pattern != NULL, NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
@@ -924,11 +925,16 @@ g_regex_new (const gchar         *pattern,
    * immediately */
   if (re == NULL)
     {
-      GError *tmp_error = g_error_new (G_REGEX_ERROR, 
-				       G_REGEX_ERROR_COMPILE,
-				       _("Error while compiling regular "
-					 "expression %s at char %d: %s"),
-				       pattern, erroffset, errmsg);
+      GError *tmp_error;
+
+      /* PCRE uses byte offsets but we want to show character offsets */
+      erroffset = g_utf8_pointer_to_offset (pattern, &pattern[erroffset]);
+
+      tmp_error = g_error_new (G_REGEX_ERROR, 
+			       G_REGEX_ERROR_COMPILE,
+			       _("Error while compiling regular "
+				 "expression %s at char %d: %s"),
+			       pattern, erroffset, errmsg);
       g_propagate_error (error, tmp_error);
 
       return NULL;
@@ -937,7 +943,8 @@ g_regex_new (const gchar         *pattern,
   /* For options set at the beginning of the pattern, pcre puts them into
    * compile options, e.g. "(?i)foo" will make the pcre structure store
    * PCRE_CASELESS even though it wasn't explicitly given for compilation. */
-  pcre_fullinfo (re, NULL, PCRE_INFO_OPTIONS, &compile_options);
+  pcre_fullinfo (re, NULL, PCRE_INFO_OPTIONS, &pcre_compile_options);
+  compile_options = pcre_compile_options;
 
   if (!(compile_options & G_REGEX_DUPNAMES))
     {
@@ -1676,7 +1683,7 @@ g_regex_split_full (const GRegex      *regex,
   i = 0;
   for (last = g_list_last (list); last; last = g_list_previous (last))
     string_list[i++] = last->data;
-  string_list[i] = 0;
+  string_list[i] = NULL;
   g_list_free (list);
 
   return string_list;
