@@ -970,7 +970,7 @@ has_more_above (const gchar *str)
 
   while (*p)
     {
-      combining_class = _g_unichar_combining_class (g_utf8_get_char (p));
+      combining_class = g_unichar_combining_class (g_utf8_get_char (p));
       if (combining_class == 230)
         return TRUE;
       else if (combining_class == 0)
@@ -1286,6 +1286,9 @@ g_unichar_get_script_bsearch (gunichar ch)
  * by Unicode Standard Annex #24). No check is made for @ch being a
  * valid Unicode character; if you pass in invalid character, the
  * result is undefined.
+ *
+ * This function is equivalent to pango_script_for_unichar() and the
+ * two are interchangeable.
  * 
  * Return value: the #GUnicodeScript for the character.
  *
