@@ -755,7 +755,8 @@ g_utf8_get_char_extended (const  gchar *p,
 /**
  * g_utf8_get_char_validated:
  * @p: a pointer to Unicode character encoded as UTF-8
- * @max_len: the maximum number of bytes to read, or -1, for no maximum.
+ * @max_len: the maximum number of bytes to read, or -1, for no maximum or
+ *           if @p is nul-terminated
  * 
  * Convert a sequence of bytes encoded as UTF-8 to a Unicode character.
  * This function checks for incomplete characters, for invalid characters
@@ -1094,7 +1095,7 @@ g_utf16_to_utf8 (const gunichar2  *str,
   gint n_bytes;
   gunichar high_surrogate;
 
-  g_return_val_if_fail (str != 0, NULL);
+  g_return_val_if_fail (str != NULL, NULL);
 
   n_bytes = 0;
   in = str;
@@ -1235,7 +1236,7 @@ g_utf16_to_ucs4 (const gunichar2  *str,
   gint n_bytes;
   gunichar high_surrogate;
 
-  g_return_val_if_fail (str != 0, NULL);
+  g_return_val_if_fail (str != NULL, NULL);
 
   n_bytes = 0;
   in = str;
@@ -1631,7 +1632,9 @@ fast_validate_len (const char *str,
   gunichar min = 0;
   const gchar *p;
 
-  for (p = str; (max_len < 0 || (p - str) < max_len) && *p; p++)
+  g_assert (max_len >= 0);
+
+  for (p = str; ((p - str) < max_len) && *p; p++)
     {
       if (*(guchar *)p < 128)
 	/* done */;
@@ -1642,7 +1645,7 @@ fast_validate_len (const char *str,
 	  last = p;
 	  if ((*(guchar *)p & 0xe0) == 0xc0) /* 110xxxxx */
 	    {
-	      if (G_UNLIKELY (max_len >= 0 && max_len - (p - str) < 2))
+	      if (G_UNLIKELY (max_len - (p - str) < 2))
 		goto error;
 	      
 	      if (G_UNLIKELY ((*(guchar *)p & 0x1e) == 0))
@@ -1655,7 +1658,7 @@ fast_validate_len (const char *str,
 	    {
 	      if ((*(guchar *)p & 0xf0) == 0xe0) /* 1110xxxx */
 		{
-		  if (G_UNLIKELY (max_len >= 0 && max_len - (p - str) < 3))
+		  if (G_UNLIKELY (max_len - (p - str) < 3))
 		    goto error;
 		  
 		  min = (1 << 11);
@@ -1664,7 +1667,7 @@ fast_validate_len (const char *str,
 		}
  	      else if ((*(guchar *)p & 0xf8) == 0xf0) /* 11110xxx */
 		{
-		  if (G_UNLIKELY (max_len >= 0 && max_len - (p - str) < 4))
+		  if (G_UNLIKELY (max_len - (p - str) < 4))
 		    goto error;
 		  
 		  min = (1 << 16);
@@ -1744,7 +1747,6 @@ g_utf8_validate (const char   *str,
     return TRUE;
 }
 
-
 /**
  * g_unichar_validate:
  * @ch: a Unicode character
@@ -1780,12 +1782,11 @@ g_unichar_validate (gunichar ch)
  * Since: 2.2
  */
 gchar *
-g_utf8_strreverse (const gchar *str, 
-		   gssize len)
+g_utf8_strreverse (const gchar *str,
+		   gssize       len)
 {
-  gchar *result;
+  gchar *r, *result;
   const gchar *p;
-  gchar *m, *r, skip;
 
   if (len < 0)
     len = strlen (str);
@@ -1793,9 +1794,9 @@ g_utf8_strreverse (const gchar *str,
   result = g_new (gchar, len + 1);
   r = result + len;
   p = str;
-  while (*p) 
+  while (r > result)
     {
-      skip = g_utf8_skip[*(guchar*)p];
+      gchar *m, skip = g_utf8_skip[*(guchar*) p];
       r -= skip;
       for (m = r; skip; skip--)
         *m++ = *p++;
